%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This script enables to reconstruct a cell cycle regulatory subnetwork of 
% Saccharomyces cerevisiae by using BANJO software and to compute the 
% corresponding  performance indexes, assuming different levels of prior 
% knowledge (PK) about the original network structure.
% To this aim, the following functions have been used:
% banjo_results.m and ppv_sens_spec.m.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% load a cell array containig the standard (first column) and systematic 
% (second column) names of genes that compose the 
% subnetwork
load Bio_Example_data_results\name_genes_subnetwork

% load the data 
load Bio_Example_data_results\expression_data_subnetwork

% load the original network to identify, obtained by the BioGRID database, 
% taking into account the information of Simon et al  (2001) and Bahler 
% (2005)
load Bio_Example_data_results\true_subnetwork


% define the number of nodes of the subnetwork
n_nodes=size(original_network,1);

n_best_net=10; % number of the highest scoring networks


% define the sparsity coefficient for the subnetwork
spar_coef=(n_nodes^2-(nnz(original_network(1:27,1:27))-...
    nnz(diag(original_network(1:27,1:27)))))/(n_nodes^2-n_nodes);


% number of tests: we assuming different levels of prior
% knowledge (0%, 20%, 50%, 70% PK) at different levels of sensitivity.
n_tests=4;

% load a cell array containing the Constraints Matrices
load Bio_Example_data_results\Constr_mtx


% allocate a memory space to save the output variables 
A_id_cell_Banjo{n_tests}=[];% cell array containing the top-scoring 
                            % networks for each test 
                                         


% define a flag (do_sim), true if you choose to do the simulation,
% otherwise you can load the files.mat, where the identified variables have 
% been saved
do_sim=false;                                     
                                    
if not(do_sim)
    % load the results of the simulations 
    load Bio_Example_data_results\results_Banjo_Bio_Example       
end



% define matrices to store the results 
PPV_Banjo=zeros(n_best_net,n_tests);
Se_Banjo=zeros(n_best_net,n_tests);
PPV_Banjo_und=zeros(n_best_net,n_tests);
Se_Banjo_und=zeros(n_best_net,n_tests);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%%%%%%%%%%%     COMPUTE THE PERFORMANCE OF BANJO   %%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for num_test=1:n_tests
    % num_test=1, corresponding to the results obtained by using 0% of
    % edges a priori known
    % num_test=2, corresponding to the results obtained by using 20% of
    % edges a priori known
    % num_test=3, corresponding to the results obtained by using 50% of
    % edges a priori known
    % num_test=4, corresponding to the results obtained by using 70% of
    % edges a priori known
    
    if num_test==1
        % Don't take into account the prior knowledge
        str_pk='_pk_0';
    elseif num_test==2
        str_pk='_pk_20';
    elseif num_test==3
        str_pk='_pk_50';
    elseif num_test==4
        str_pk='_pk_70';
    end
    
    
    if do_sim
        % Compute the top-scoring networks corresponding to the
        % connectivity matrices
        eval(['dos(''java -jar banjo.jar settingsFile=data\'...
            'release2.0\dynamic\Banjo_dyn_sett_file_Bio_Example\'...
            'dyn_sett_sys_Bio_Example',str_pk,'.txt'')']);
        % define a string containing the path of the report file
        % returned by Banjo
        filename=['data\release2.0\dynamic\output'...
            '\Banjo_results_Bio_Example'...
            '\results_Bio_Example',str_pk,'.txt'];
        
        % Obtain the connectivity matrices, corresponding to the highest 
        % scoring networks returned by report file of Banjo 
        [A_id_best_Banjo]=banjo_results(filename,n_nodes,...
                n_best_net,false);

        % store the cell array that contains the best identified
        % connectivity matrices for each test
        A_id_cell_Banjo{num_test}=A_id_best_Banjo;
    else
        % Assign the connectivity matrices, corresponding to the highest 
        % scoring networks obtained by Banjo, for each test
        A_id_best_Banjo=A_id_cell_Banjo{num_test};        
    end
        
    % evaluation of the performance for the ten highest scoring networks by 
    % calculating the PPV and sensitivity indexes
    for num_iter=1:n_best_net; 

        Perf_idx= ppv_sens_spec(original_network,...
            A_id_best_Banjo{num_iter});

        % for the performance evaluation, consider the case of directed
        % graph representations: the network is represented as a
        % directed graph, the performance indexes (PPV, Sensitivity) do
        % not take into account the estimated weight of the edge, but
        % only its existence;
        % store in two matrix (PPV_pa, Se_pa) the PPV and Se performance
        % indexes: the row index corresponds to the iteration number of the
        % iterative procedure, the column index corresponds to the kind of 
        % test
        PPV_Banjo(num_iter,num_test)=Perf_idx(2,1);
        Se_Banjo(num_iter,num_test)=Perf_idx(2,2);
        
        if num_test==1
            % To allow a comparison with CLR, with no prior knowledge,
            % consider that the network is represented by graph, then
            % for the performance evaulation, the connectivity matrix
            % is undirected; store in two matrix (PPV_Banjo_und and
            % Se_Banjo_und) the PPV and Se performance indexes
            PPV_Banjo_und(num_iter,num_test)=Perf_idx(1,1);
            Se_Banjo_und(num_iter,num_test)=Perf_idx(1,2);
        end
    end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%    PLOT RESULTS WITHOUT PK  %%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Compute the performance of random algorithm without take into account the
% prior knowledge (PK)
PK_l=0;
% Vector of reconstruction probability values
% Each value defines the probability that an element of the randomly 
% reconstructed matrix is zero
x_pr_v=linspace(0,1,100);

% Compute the corresponding vectors of PPV and Sens, for undirected 
% randomly reconstructed networks
TP_und=PK_l*(1-spar_coef.^2)+(1-PK_l)*(1-spar_coef.^2)*(1-x_pr_v.^2);
% TN_und=x_pr_v.^2*spar_coef^2;
FP_und=(1-x_pr_v.^2)*spar_coef^2;
FN_und=(1-PK_l)*x_pr_v.^2*(1-spar_coef^2);

sens_und=TP_und./(TP_und+FN_und);
% spec_und=TN_und./(TN_und+FP_und);
PPV_und=TP_und./(TP_und+FP_und);


% Compute the corresponding vectors of PPV and Sens, for directed 
% randomly reconstructed networks
TP_dir=PK_l*(1-spar_coef)+(1-PK_l)*(1-spar_coef)*(1-x_pr_v);
%TN_dir=x_pr_v*spar_coef;
FP_dir=(1-x_pr_v)*spar_coef;
FN_dir=(1-PK_l)*x_pr_v*(1-spar_coef);

sens_dir=TP_dir./(TP_dir+FN_dir);
% spec_dir=TN_dir./(TN_dir+FP_dir);
PPV_dir=TP_dir./(TP_dir+FP_dir);

figure;
hold on;
grid on
title('UND 0%PK','FontSize', 16)
xlabel('Sn','FontSize', 14)
ylabel('PPV','FontSize', 14)
axis([0 0.6 0 0.8]);

% plot the performance of the random algorithm (UNDIRECTED representation)
plot(sens_und,PPV_und,'r','LineWidth',2);

% plot the performance of Banjo, with 0% level of
% PK (UNDIRECTED representation)
plot(Se_Banjo_und(:,1),PPV_Banjo_und(:,1),'sk','MarkerSize',10,...
     'LineWidth',2);
legend('RANDOM','BANJO','FontSize', 16)

figure;
hold on;
grid on
title('DIR 0%PK','FontSize', 16)
xlabel('Sn','FontSize', 14)
ylabel('PPV','FontSize', 14)
axis([0 0.4 0 0.5]);
% plot the performance of the random algorithm (DIRECTED representation)
plot(sens_dir,PPV_dir,'r','LineWidth',2);

% plot the performance of Banjo, with 0% level of
% PK (DIRECTED representation)
plot(Se_Banjo(:,1),PPV_Banjo(:,1),'sk','MarkerSize',10,...
    'LineWidth',2);
legend('RANDOM','BANJO','FontSize', 16)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%  PLOT RESULTS WITH PK  %%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Vector of reconstruction probability values
% Each value defines the probability that an element of the randomly 
% reconstructed matrix is zero
x_pr_v=linspace(0,1,100);

% Compute the performance of random algorithm taking into account 20% of 
% edges a priori known
PK_l=0.2;

% Compute the corresponding vectors of PPV and Sens, for directed 
% randomly reconstructed networks
TP_dir_20=PK_l*(1-spar_coef)+(1-PK_l)*(1-spar_coef)*(1-x_pr_v);
%TN_dir_20=x_pr_v*spar_coef;
FP_dir_20=(1-x_pr_v)*spar_coef;
FN_dir_20=(1-PK_l)*x_pr_v*(1-spar_coef);
sens_dir_20=TP_dir_20./(TP_dir_20+FN_dir_20);
% spec_dir_20=TN_dir./(TN_dir_20+FP_dir_20);
PPV_dir_20=TP_dir_20./(TP_dir_20+FP_dir_20);

% Compute the performance of random algorithm taking into account 50% of 
% edges a priori known
PK_l=0.5;

% Compute the corresponding vectors of PPV and Sens, for directed 
% randomly reconstructed networks
TP_dir_50=PK_l*(1-spar_coef)+(1-PK_l)*(1-spar_coef)*(1-x_pr_v);
%TN_dir_50=x_pr_v*spar_coef;
FP_dir_50=(1-x_pr_v)*spar_coef;
FN_dir_50=(1-PK_l)*x_pr_v*(1-spar_coef);
sens_dir_50=TP_dir_50./(TP_dir_50+FN_dir_50);
% spec_dir_50=TN_dir./(TN_dir_50+FP_dir_50);
PPV_dir_50=TP_dir_50./(TP_dir_50+FP_dir_50);

% Compute the performance of random algorithm taking into account 70% of 
% edges a priori known
PK_l=0.7;

% Compute the corresponding vectors of PPV and Sens, for directed 
% randomly reconstructed networks
TP_dir_70=PK_l*(1-spar_coef)+(1-PK_l)*(1-spar_coef)*(1-x_pr_v);
%TN_dir_70=x_pr_v*spar_coef;
FP_dir_70=(1-x_pr_v)*spar_coef;
FN_dir_70=(1-PK_l)*x_pr_v*(1-spar_coef);
sens_dir_70=TP_dir_70./(TP_dir_70+FN_dir_70);
% spec_dir_70=TN_dir./(TN_dir_70+FP_dir_70);
PPV_dir_70=TP_dir_70./(TP_dir_70+FP_dir_70);


figure
hold on;
grid on
title('BANJO with PK','FontSize', 16)
xlabel('Sn','FontSize', 14)
ylabel('PPV','FontSize', 14)
axis([0.2 1 0.1 1]);


% plot the performance of BANJO and of the random algorithm 
% assuming 3 different levels of PK:

% 1) 20% of PK
plot(Se_Banjo(:,2),PPV_Banjo(:,2),'sk','MarkerSize',8,...
    'LineWidth',2);
plot(sens_dir_20,PPV_dir_20,'r:','LineWidth',2);

% 2) 50% of PK
plot(Se_Banjo(:,3),PPV_Banjo(:,3),'sk','MarkerSize',8,...
    'LineWidth',2);
plot(sens_dir_50,PPV_dir_50,'r--','LineWidth',2);

% 3) 70% of PK
plot(Se_Banjo(:,4),PPV_Banjo(:,4),'sk','MarkerSize',8,...
    'LineWidth',2);
plot(sens_dir_70,PPV_dir_70,'r-','LineWidth',2);

