%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This script enables to reconstruct a cell cycle regulatory subnetwork of 
% Saccharomyces cerevisiae and to compute the performance indexes by using 
% the CLR software.
% To this aim, the following functions have been used:
% clr.m, matrixPvalue_mod.m and matrixCompare_mod.m.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% load a cell array containig the standard (first column) and systematic 
% (second column) names of genes that compose the 
% subnetwork
load Bio_Example_data_results\name_genes_subnetwork

% load the data 
load Bio_Example_data_results\expression_data_subnetwork

% load the original network to identify, obtained by the BioGRID database, 
% taking into account the information of Simon et al  (2001) and Bahler 
% (2005)
load Bio_Example_data_results\true_subnetwork

% define the number of nodes of the subnetwork
n_nodes=size(original_network,1);


% Apply a smoothing technique in order to filter the measurement noise
% and increase by interpolation the number of observations
Meas_mtx_smth=smth_int_data(Meas_mtx_dec,3,0);

% define the sparsity coefficient for the subnetwork
spar_coef=(n_nodes^2-(nnz(original_network(1:27,1:27))-...
    nnz(diag(original_network(1:27,1:27)))))/(n_nodes^2-n_nodes);

% % allocate a memory space to save the output variables 
% A_id_cell_clr{1}=[]; % cell array containing the reconstructed network
% 
% define a flag (do_sim), true if you choose to do the simulation,
% otherwise you can load the files.mat, where the identified variables have 
% been saved
do_sim=false;                                     
                                    
if not(do_sim)
    load Bio_Example_data_results\results_Clr_Bio_Example
end
                                       

% define start..incr..finish - threshold region to sample
start=1;
incr=1;
finish=100;
precCutoff=5;
n_sample=length(start:incr:finish);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%%%%%%%%%%%%%  COMPUTE THE PERFORMANCE OF CLR  %%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


if do_sim
    % Compute the reconstructed network corresponding to the
    % connectivity matrix via CLR
    [Astouffer, MI] = clr(Meas_mtx_smth', 'stouffer', 10, 3);
    A_id_plos = clr(MI, 'plos');
%     % store the cell array that contains the identified
%     % connectivity matrix
%     A_id_cell_clr{1}=A_id_plos;
end

% evaluation of the performance by calculating the PPV and
% sensitivity indexes (so we commment some code of the original
% function matrixPvalue)
% We do not take into account the elements on diagonal, so we modified
% the original function matrixCompare
orig_net=original_network;
for index=1:n_nodes
    orig_net(index,index)=0;
end
% consider that the network is represented by graph, then for
% the performance evaulation, the connectivity matrix is
% undirected; store in two vectors (prec and sens) the PPV and Se performance
% indexes for different values of sparsity of the identified
% connectivity matrix
        
[prec, sens] = matrixPvalue_mod(A_id_plos, orig_net, n_nodes,...
    start,incr,finish,precCutoff);
        
        
% Compute the performance of random algorithm without take into account the
% prior knowledge (PK)
PK_l=0;
% Vector of reconstruction probability values
% Each value defines the probability that an element of the randomly 
% reconstructed matrix is zero
x_pr_v=linspace(0,1,100);

% Compute the corresponding vectors of PPV and Sens, for undirected 
% randomly reconstructed networks
TP_und=PK_l*(1-spar_coef.^2)+(1-PK_l)*(1-spar_coef.^2)*(1-x_pr_v.^2);
% TN_und=x_pr_v.^2*spar_coef^2;
FP_und=(1-x_pr_v.^2)*spar_coef^2;
FN_und=(1-PK_l)*x_pr_v.^2*(1-spar_coef^2);

sens_und=TP_und./(TP_und+FN_und);
% spec_und=TN_und./(TN_und+FP_und);
PPV_und=TP_und./(TP_und+FP_und);

figure;
hold on;
grid on
title('UND 0%PK','FontSize', 16)
xlabel('Sn','FontSize', 14)
ylabel('PPV','FontSize', 14)
axis([0 0.6 0 0.8]);
% plot the performance of the random algorithm (UNDIRECTED representation)
plot(sens_und,PPV_und,'r','LineWidth',2);
% plot the performance of CLR
plot(sens(1:3:end)', prec(1:3:end)','vm','MarkerSize',10,'LineWidth',2);

legend('RANDOM','CLR','FontSize', 16)

