%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This script enables to reconstruct a cell cycle regulatory subnetwork of 
% Saccharomyces cerevisiae. Such network is composed of 27 genes: ten genes 
% that encoded the corresponding transcription factors proteins (ace2, 
% fkh1, swi4, swi5, mbp1, swi6, mcm1, fkh2, ndd1, yox1) and seventeen genes 
% that encoded corresponding cyclin and cyclin/CDK regulator proteins 
% (cln1, cln2, cln3, cdc20, clb1, clb2, clb4, clb5, clb6, sic1, far1, 
% spo12, apc1, tem1, gin4, swe1 and whi5). We have used the microarray data 
% from Spellman et al 1998 and selected the data set produced by alfa 
% factor arrest method. Thus we have obtained a data set of 27 genes and 18 
% data points. The regulatory network between the 27 genes was drawn from 
% the BioGRID database, taking into account the information of Simon et al 
% (2001) and Bahler (2005). To reconstuct the network and to compute the 
% performance indexes, the following functions have been used:
% main_corenet.m and main_corenet_nopa.m that recall 
% lmi_sign_constr_full_ex.m and lmi_sign_constr_ex.m; ppv_sens_spec.m.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% load a cell array containig the standard (first column) and systematic 
% (second column) names of genes that compose the 
% subnetwork
load Bio_Example_data_results\name_genes_subnetwork

% load the data 
load Bio_Example_data_results\expression_data_subnetwork

% load the original network to identify, obtained by the BioGRID database, 
% taking into account the information of Simon et al  (2001) and Bahler 
% (2005). 
load Bio_Example_data_results\true_subnetwork



% define the maxinum number of iterations for each identification
max_iter_num_no_pk=120;
max_iter_num_pk=20;

% threshold value for the estimation error 
thr_err=0.1;

% define the number of nodes of the subnetwork
n_nodes=size(original_network,1);


% Apply a smoothing technique in order to filter the measurement noise
% and increase by interpolation the number of observations
Meas_mtx_smth=smth_int_data(Meas_mtx_dec,3,0);

% define the sparsity coefficient for the subnetwork
spar_coef=(n_nodes^2-(nnz(original_network(1:27,1:27))-...
    nnz(diag(original_network(1:27,1:27)))))/(n_nodes^2-n_nodes);

% Assume that B vector is null
B_vct=zeros(n_nodes,1);

% number of tests: we assuming different levels of prior
% knowledge (0%, 20%, 50%, 70% PK) at different levels of sensitivity.
n_tests=4;

% load a cell array containing the Constraints Matrices
load Bio_Example_data_results\Constr_mtx


% allocate a memory space to save the output variables of the iterative
% procedure
A_id_cell_pa{n_tests}=[]; % cell array containing the identified 
                          % connectivity matrices, obtained by
                          % using CORE-Net, for each test  

% U_id_cell_pa{n_tests}=[]; % cell array containing the identified input 
                            % vectors, obtained by
                            % using CORE-Net, for each test  

est_err_cell_pa{n_tests}=[]; % cell array containing the errors of the 
                             % identified models, obtained by
                             % using CORE-Net, for each test 
                             
sol_found_pa{n_tests}=[]; % cell array containing a flag, true if a 
                          % solution has been found, obtained by
                          % using CORE-Net, for each test  

A_id_cell_std{n_tests}=[]; % cell array containing the identified
                           % connectivity matrices, obtained by
                           % using CORE-Net_noPA, for each test  

                                    
% U_id_cell_std{n_tests}=[]; % cell array containing the identified input 
                             % vectors, obtained by
                             % using CORE-Net_noPA, for each test  
                                    
est_err_cell_std{n_tests}=[]; % cell array containing the errors of the  
                              % identified models, obtained by
                              % using CORE-Net_noPA, for each test  
                                        
sol_found_std{n_tests}=[]; % cell array containing a flag, true if a 
                           % solution has been found, obtained by
                           % using CORE-Net_noPA, for each test  

% define a flag (do_sim), true if you choose to do the simulation,
% otherwise you can load the files.mat, where the identified variables have 
% been saved
do_sim=false;                                     
                                    
if not(do_sim)
    % load the results of the simulations 
%     load Bio_Example_data_results\results_subnetwork  
    load Bio_Example_data_results\results_corenet_Bio_Example
end



% define matrices to store the results 
PPV_pa=zeros(max_iter_num_no_pk,n_tests);
Se_pa=zeros(max_iter_num_no_pk,n_tests);
PPV_std=zeros(max_iter_num_no_pk,n_tests);
Se_std=zeros(max_iter_num_no_pk,n_tests);
PPV_pa_und=zeros(max_iter_num_no_pk,n_tests);
Se_pa_und=zeros(max_iter_num_no_pk,n_tests);
PPV_std_und=zeros(max_iter_num_no_pk,n_tests);
Se_std_und=zeros(max_iter_num_no_pk,n_tests);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%    COMPUTE THE PERFORMANCE OF CORE-Net and CORE-Net_noPA    %%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for num_test=1:n_tests
    if num_test==1
        max_iter_num=max_iter_num_no_pk;
    else
        max_iter_num=max_iter_num_pk;
    end
    % num_test=1, corresponding to the results obtained by using 0% of
    % edges a priori known
    % num_test=2, corresponding to the results obtained by using 20% of
    % edges a priori known
    % num_test=3, corresponding to the results obtained by using 50% of
    % edges a priori known
    % num_test=4, corresponding to the results obtained by using 70% of
    % edges a priori known
    
    if do_sim
        % Compute the variables to be identified (the connectivity matrix 
        % A)

        % Inference by using CORE-Net
        [A_id_cell,U_id_cell,est_err_cell,sol_found] = ...
            main_corenet(Meas_mtx_smth,B_vct,Constr_mtx{num_test},...
            n_nodes,max_iter_num);

        % store the identified variables
        A_id_cell_pa{num_test}=A_id_cell;
        % U_id_cell_pa{num_test}=U_id_cell;
        est_err_cell_pa{num_test}=est_err_cell;
        sol_found_pa{num_test}=sol_found;
    else
        A_id_cell=A_id_cell_pa{num_test};
        % U_id_cell=U_id_cell_pa{num_net,num_test};
        % est_err_cell=est_err_cell_pa{num_test};
    end
    
    % evaluation of the performance at different levels of Sensitivity (Se)
    for num_iter=1:max_iter_num
        if num_iter<=length(A_id_cell)
            Perf_idx= ppv_sens_spec(original_network,A_id_cell{num_iter});

            % for the performance evaluation, consider the case of directed
            % graph representations: the network is represented as a
            % directed graph, the performance indexes (PPV, Sensitivity) do
            % not take into account the estimated weight of the edge, but
            % only its existence; store in two matrix (PPV_pa, Se_pa) the
            % PPV and Se performance indexes: the row index corresponds to
            % the iteration number of the iterative procedure, the column
            % index corresponds to the test
            PPV_pa(num_iter,num_test)=Perf_idx(2,1);
            Se_pa(num_iter,num_test)=Perf_idx(2,2);

            if num_test==1
                % To allow a comparison with CLR, with no prior knowledge,
                % consider that the network is represented by graph, then
                % for the performance evaulation, the connectivity matrix
                % is undirected; store in two matrix (PPV_pa_und, Se_pa_und
                % the PPV and Se performance indexes
                PPV_pa_und(num_iter,num_test)=Perf_idx(1,1);
                Se_pa_und(num_iter,num_test)=Perf_idx(1,2);
            end
        end
    end


    if do_sim
        % Compute the variables to be identified (the connectivity matrix 
        % A)
        % Inference by using CORE-Net_noPA
        [A_id_cell,U_id_cell,est_err_cell,sol_found] = ...
            main_corenet_nopa(Meas_mtx_smth,B_vct,Constr_mtx{num_test},...
            n_nodes,max_iter_num);

        % store the identified variables
        A_id_cell_std{num_test}=A_id_cell;
        % U_id_cell_std{num_test}=U_id_cell;
        est_err_cell_std{num_test}=est_err_cell;
        sol_found_std{num_test}=sol_found;
    else
        A_id_cell=A_id_cell_std{num_test};
        % U_id_cell=U_id_cell_std{num_net,num_test};
        % est_err_cell=est_err_cell_std{num_net,num_test};
    end

    % evaluation of the performance at different levels of Sensitivity (Se) 
    for num_iter=1:max_iter_num
       if num_iter<=length(A_id_cell)
            Perf_idx= ppv_sens_spec(original_network,A_id_cell{num_iter});

            % for the performance evaluation, consider the case of directed
            % graph representations: the network is represented as a
            % directed graph, the performance indexes (PPV, Sensitivity) do
            % not take into account the estimated weight of the edge, but
            % only its existence; store in two matrix (PPV_std, Se_std) the
            % PPV and Se performance indexes: the row index corresponds to
            % the iteration number of the iterative procedure, the column
            % index corresponds to the test
            PPV_std(num_iter,num_test)=Perf_idx(2,1);
            Se_std(num_iter,num_test)=Perf_idx(2,2);

            if num_test==1
                % To allow a comparison with CLR, with no prior knowledge,
                % consider that the network is represented by graph, then
                % for the performance evaulation, the connectivity matrix
                % is undirected; store in two matrix (PPV_std_und and
                % Se_std_und) the PPV and Se performance indexes
                PPV_std_und(num_iter,num_test)=Perf_idx(1,1);
                Se_std_und(num_iter,num_test)=Perf_idx(1,2);
            end
       end
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%    PLOT RESULTS WITHOUT PK  %%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



% Compute the performance of random algorithm without take into account the
% prior knowledge (PK)
PK_l=0;
% Vector of reconstruction probability values
% Each value defines the probability that an element of the randomly 
% reconstructed matrix is zero
x_pr_v=linspace(0,1,100);

% Compute the corresponding vectors of PPV and Sens, for undirected 
% randomly reconstructed networks
TP_und=PK_l*(1-spar_coef.^2)+(1-PK_l)*(1-spar_coef.^2)*(1-x_pr_v.^2);
% TN_und=x_pr_v.^2*spar_coef^2;
FP_und=(1-x_pr_v.^2)*spar_coef^2;
FN_und=(1-PK_l)*x_pr_v.^2*(1-spar_coef^2);

sens_und=TP_und./(TP_und+FN_und);
% spec_und=TN_und./(TN_und+FP_und);
PPV_und=TP_und./(TP_und+FP_und);

% Compute the corresponding vectors of PPV and Sens, for directed 
% randomly reconstructed networks
TP_dir=PK_l*(1-spar_coef)+(1-PK_l)*(1-spar_coef)*(1-x_pr_v);
%TN_dir=x_pr_v*spar_coef;
FP_dir=(1-x_pr_v)*spar_coef;
FN_dir=(1-PK_l)*x_pr_v*(1-spar_coef);

sens_dir=TP_dir./(TP_dir+FN_dir);
% spec_dir=TN_dir./(TN_dir+FP_dir);
PPV_dir=TP_dir./(TP_dir+FP_dir);


figure;
hold on;
grid on
title('UND 0%PK','FontSize', 16)
xlabel('Sn','FontSize', 14)
ylabel('PPV','FontSize', 14)
axis([0 0.6 0 0.8]);

% plot the performance of the random algorithm (UNDIRECTED representation)
plot(sens_und,PPV_und,'r','LineWidth',2);

% plot the performance of the algorithms (CORE-Net and CORE-Net_noPA), 
% with 0% level of PK (UNDIRECTED representation)
plot(Se_pa_und([2:8,10:5:80],1),PPV_pa_und([2:8,10:5:80],1),'MarkerFaceColor',[0 0 1],...
    'MarkerSize',10,'Marker','o','LineWidth',2,'LineStyle','none');
plot(Se_std_und([2:8,10:10:120],1),PPV_std_und([2:8,10:10:120],1),'^g','MarkerSize',10,...
    'LineWidth',2);
legend('RANDOM','CORE-Net','CORE-Net_{noPA}','FontSize', 16)


% plot(Se_pa_und(2:80,1),PPV_pa_und(2:80,1),'MarkerFaceColor',[0 0 1],...
%     'MarkerSize',10,'Marker','o','LineWidth',2,'LineStyle','none');
% plot(Se_std_und(2:80,1),PPV_std_und(2:80,1),'^g','MarkerSize',10,...
%     'LineWidth',2);

figure;
hold on;
grid on
title('DIR 0%PK','FontSize', 16)
xlabel('Sn','FontSize', 14)
ylabel('PPV','FontSize', 14)
axis([0 0.4 0 0.5]);
% plot the performance of the random algorithm (DIRECTED representation)
plot(sens_dir,PPV_dir,'r','LineWidth',2);
% plot the performance of the algorithms (CORE-Net and CORE-Net_noPA), with
% 0% level of PK (DIRECTED representation)
legend('RANDOM','CORE-Net','CORE-Net_{noPA}','FontSize', 16)


% plot the performance of the algorithms (CORE-Net and CORE-Net_noPA), with
% 0% level of PK (DIRECTED representation)
% plot(Se_pa(2:80,1),PPV_pa(2:80,1),'MarkerFaceColor',[0 0 1],...
%     'MarkerSize',10,'Marker','o','LineWidth',2,'LineStyle','none');
% plot(Se_std(2:80,1),PPV_std(2:80,1),'^g','MarkerSize',10,...
%     'LineWidth',2);
plot(Se_pa([2:8,10:5:80],1),PPV_pa([2:8,10:5:80],1),'MarkerFaceColor',[0 0 1],...
    'MarkerSize',10,'Marker','o','LineWidth',2,'LineStyle','none');
plot(Se_std([2:8,10:5:80],1),PPV_std([2:8,10:5:80],1),'^g','MarkerSize',10,...
    'LineWidth',2);
legend('RANDOM','CORE-Net','CORE-Net_{noPA}','FontSize', 16)




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%  PLOT RESULTS WITH PK  %%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Vector of reconstruction probability values
% Each value defines the probability that an element of the randomly 
% reconstructed matrix is zero
x_pr_v=linspace(0,1,100);

% Compute the performance of random algorithm taking into account 20% of 
% edges a priori known
PK_l=0.2;

% Compute the corresponding vectors of PPV and Sens, for directed 
% randomly reconstructed networks
TP_dir_20=PK_l*(1-spar_coef)+(1-PK_l)*(1-spar_coef)*(1-x_pr_v);
%TN_dir_20=x_pr_v*spar_coef;
FP_dir_20=(1-x_pr_v)*spar_coef;
FN_dir_20=(1-PK_l)*x_pr_v*(1-spar_coef);
sens_dir_20=TP_dir_20./(TP_dir_20+FN_dir_20);
% spec_dir_20=TN_dir./(TN_dir_20+FP_dir_20);
PPV_dir_20=TP_dir_20./(TP_dir_20+FP_dir_20);

% Compute the performance of random algorithm taking into account 50% of 
% edges a priori known
PK_l=0.5;

% Compute the corresponding vectors of PPV and Sens, for directed 
% randomly reconstructed networks
TP_dir_50=PK_l*(1-spar_coef)+(1-PK_l)*(1-spar_coef)*(1-x_pr_v);
%TN_dir_50=x_pr_v*spar_coef;
FP_dir_50=(1-x_pr_v)*spar_coef;
FN_dir_50=(1-PK_l)*x_pr_v*(1-spar_coef);
sens_dir_50=TP_dir_50./(TP_dir_50+FN_dir_50);
% spec_dir_50=TN_dir./(TN_dir_50+FP_dir_50);
PPV_dir_50=TP_dir_50./(TP_dir_50+FP_dir_50);

% Compute the performance of random algorithm taking into account 70% of 
% edges a priori known
PK_l=0.7;

% Compute the corresponding vectors of PPV and Sens, for directed 
% randomly reconstructed networks
TP_dir_70=PK_l*(1-spar_coef)+(1-PK_l)*(1-spar_coef)*(1-x_pr_v);
%TN_dir_70=x_pr_v*spar_coef;
FP_dir_70=(1-x_pr_v)*spar_coef;
FN_dir_70=(1-PK_l)*x_pr_v*(1-spar_coef);
sens_dir_70=TP_dir_70./(TP_dir_70+FN_dir_70);
% spec_dir_70=TN_dir./(TN_dir_70+FP_dir_70);
PPV_dir_70=TP_dir_70./(TP_dir_70+FP_dir_70);


figure
hold on
grid on
title('CORE-Net and CORE-Net_{noPA} with PK','FontSize', 16)
xlabel('Sn','FontSize', 14)
ylabel('PPV','FontSize', 14)
axis([0.2 1 0.1 1]);

% plot the performance of the algorithms (CORE-Net, CORE-Net_noPA and random algorithm), 
% with different levels of PK (DIRECTED representation)

% 1) 20% of PK
plot(Se_pa(1:max_iter_num_pk,2),PPV_pa(1:max_iter_num_pk,2),'MarkerFaceColor',[0 0 1],...
    'MarkerSize',8,'Marker','o','LineWidth',2,'LineStyle','none');
plot(Se_std(1:max_iter_num_pk,2),PPV_std(1:max_iter_num_pk,2),'^g','MarkerSize',8,...
    'LineWidth',2);
plot(sens_dir_20,PPV_dir_20,'r:','LineWidth',2);

% 2) 50% of PK
plot(Se_pa(1:9,3),PPV_pa(1:9,3),'MarkerFaceColor',[0 0 1],...
    'MarkerSize',8,'Marker','o','LineWidth',2,'LineStyle','none');
plot(Se_std(1:9,3),PPV_std(1:9,3),'^g','MarkerSize',8,...
    'LineWidth',2);
plot(sens_dir_50,PPV_dir_50,'r--','LineWidth',2);

% 3) 70% of PK
plot(Se_pa(1:12,4),PPV_pa(1:12,4),'MarkerFaceColor',[0 0 1],...
    'MarkerSize',8,'Marker','o','LineWidth',2,'LineStyle','none');
plot(Se_std(1:9,4),PPV_std(1:9,4),'^g','MarkerSize',8,...
    'LineWidth',2);
plot(sens_dir_70,PPV_dir_70,'r-','LineWidth',2);



