%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This script enables to compute the reconstruced connectivity matrices and
% performance indexes for the in-silico networks of ten and twenty nodes, 
% generated by using the scalefree network generation model proposed by 
% Albert and Barabasi, via the BANJO software. For each network, different 
% noise levels and different levels of prior knowledge (PK) about the
% original network structure have been assumed.
% To this aim, the following functions have been used:
% banjo_results.m and ppv_sens_spec.m.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% select which networks you want to test
net_nodes=10; % test over the networks of ten nodes
% net_nodes=20; % test over the networks of twenty nodes 

% networks number
n_networks= 20;

n_best_net=5; % number of the highest scoring networks

% number of tests: for each network, we consider a set of data of 20 time 
% points, assuming there levels of noise (0%, 10%, 30%) and two levels of 
% prior knowledge (0% and 30%)
n_tests=6;

% allocate a memory space to save the output variables 
A_id_cell_Banjo{n_networks,n_tests}=[]; % cell array containing the 
                                        % top-scoring network, for each 
                                        % test  

% define a flag (do_sim), true if you choose to do the simulation,
% otherwise you can load the files.mat, where the identified variables have 
% been saved
do_sim=false;                                     
                                    
if not(do_sim)
    % load the results of the simulations 
    if net_nodes==10
        load results_net_in_silico_10n\results_Banjo_10n
    else    
        load results_net_in_silico_20n\results_Banjo_20n
    end
end
                                       
% define a vector to store the sparsity coefficient for each network
spars_networks=zeros(1,n_networks);


% define matrices to store the results 
PPV_Banjo=zeros(n_networks,n_tests);
Se_Banjo=zeros(n_networks,n_tests);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%%%%%%%%%%%%%  COMPUTE THE PERFORMANCE OF BANJO  %%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for num_net=1:n_networks 
    % load the data for the current network
    if net_nodes==10
        eval(['load data_net_in_silico_10n\sys_in_silico_10n_net_',...
            num2str(num_net)]);
    else
        eval(['load data_net_in_silico_20n\sys_in_silico_20n_net_',...
            num2str(num_net)]);
    end
    spars_networks(num_net)=spar_coef;
    
    for num_test=1:n_tests
        % num_test=1, corresponding to the results obtained by using a data
        %             set of 20 time points, 0% level of noise, 0% level of
        %             prior knowledge             
        % num_test=2, corresponding to the results obtained by filtering
        %             and interpolating a data set of 20 time points, 10%
        %             level of noise, and using 0% level of prior
        %             knowledge 
        % num_test=3, corresponding to the results obtained by filtering
        %             and interpolating a data set of 20 time points, 30%
        %             level of noise, and using 0% level of prior
        %             knowledge 
        % num_test=4, corresponding to the results obtained by using a data
        %             set of 20 time points, 0% level of noise, 30% level 
        %             of prior knowledge 
        % num_test=5, corresponding to the results obtained by filtering
        %             and interpolating a data set of 20 time points, 10%
        %             level of noise, and using 30% level of prior
        %             knowledge 
        % num_test=6, corresponding to the results obtained by filtering
        %             and interpolating a data set of 20 time points, 30%
        %             level of noise, and using 30% level of prior
        %             knowledge 
        
        % define a string if the prior knowledge is used
        if num_test<=3
            % Don't take into account the prior knowledge
            flag_pk=false;
            str_pk='';
        else
            flag_pk=true;
            str_pk='_pk';
        end

        % define a string about the noise level of the data
        if num_test==1 || num_test==4
            % the measurements are without noise
            str_level_noise='nonoise';
        elseif num_test==2 || num_test==5
            % the measurements are with 10% level of noise
            str_level_noise='noise10';
        elseif num_test==3 || num_test==6
            % the measurements are with 30% level of noise
            str_level_noise='noise30';
        end
        
        if do_sim
            % Compute the top-scoring network corresponding to the
            % connectivity matrix
            eval(['dos(''java -jar banjo.jar settingsFile=data\'...
                'release2.0\dynamic\Banjo_dyn_sett_file_net_in_silico_',...
                num2str(net_nodes),'n\dyn_sett_sys_',...
            num2str(net_nodes),'n_data_',str_level_noise,str_pk,...
            '_net_',num2str(num_net),'.txt'')'])
            % define a string containing the path of the report file 
            % returned by Banjo        
            filename=['data\release2.0\dynamic\output'...
               '\Banjo_results_net_in_silico_',num2str(net_nodes),'n'...
               '\results_',num2str(net_nodes),'n_data_',...
            str_level_noise,str_pk,'_net_',num2str(num_net),'.txt']; 
        
            [A_id_best_Banjo]=banjo_results(filename,n_nodes,...
                n_best_net,false);
                
            % store the cell array that contains the best identified
            % connectivity matrices for each network and test
            A_id_cell_Banjo{num_net,num_test}=A_id_best_Banjo;
        else
            A_id_best_Banjo=A_id_cell_Banjo{num_net,num_test};
            
        end
        
        % evaluation of the performance by calculating the PPV and
        % sensitivity indexes for the top-scoring network
        Perf_idx= ppv_sens_spec(original_network,A_id_best_Banjo{1});
        
        % for the performance evaluation, consider the case of directed
        % graph representations: the network is represented as a
        % directed graph, the performance indexes (PPV, Sensitivity) do
        % not take into account the estimated weight of the edge, but
        % only its existence;
        % save in two matrix (PPV_Banjo, Se_Banjo) the PPV and Sensitivity
        % (Se) performance indexes: the row index corresponds to the
        % network number, the column index corresponds to the kind of
        % test
        PPV_Banjo(num_net,num_test)=Perf_idx(2,1);
        Se_Banjo(num_net,num_test)=Perf_idx(2,2);

    end
end


% Compute the performance of random algorithm without take into account the
% prior knowledge (PK)

% define a mean sparsity coefficient
spar_coef_mean=mean(spars_networks);
PK_f=0;

% Vector of reconstruction probability values
% Each value defines the probability that an element of the randomly 
% reconstructed matrix is zero
x_pr_v=linspace(0,1,100);

% Compute the corresponding vectors of PPV and Sens, for directed 
% randomly reconstructed networks

TP_dir=PK_f*(1-spar_coef_mean)+(1-PK_f)*(1-spar_coef_mean)*(1-x_pr_v);
%TN_dir=x_pr_v*spar_coef_mean;
FP_dir=(1-x_pr_v)*spar_coef_mean;
FN_dir=(1-PK_f)*x_pr_v*(1-spar_coef_mean);

sens_dir=TP_dir./(TP_dir+FN_dir);
% spec_dir=TN_dir./(TN_dir+FP_dir);
PPV_dir=TP_dir./(TP_dir+FP_dir);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%    PLOT RESULTS WITHOUT PK  %%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure;
hold on;
grid on
if net_nodes==10
    title('n=10, 0%PK','FontSize', 16)
    axis([0.1 0.3 0.2 0.55]);
elseif net_nodes==20
    title('n=20, 0%PK','FontSize', 16)
    axis([0.02 0.15 0.2 0.55]);
else
    title('0%PK','FontSize', 16)
end
xlabel('Sn','FontSize', 14)
ylabel('PPV','FontSize', 14)
% axis([0.05 0.4 0.2 0.65]);
% plot the performance of the random algorithm
plot(sens_dir,PPV_dir,'r','LineWidth',2);

% plot the performance of BANJO, averaged of a 
% set of 20 networks of the algorithm, assuming 3 different level of noise: 

% 1) 0% level of noise
plot(mean(Se_Banjo(:,1)),mean(PPV_Banjo(:,1)),'ok','MarkerSize',10,...
    'LineWidth',2);

% 2) 10% level of noise
plot(mean(Se_Banjo(:,2)),mean(PPV_Banjo(:,2)),'sk','MarkerSize',10,...
    'LineWidth',2);

% 3) 30% level of noise
plot(mean(Se_Banjo(:,3)),mean(PPV_Banjo(:,3)),'dk','MarkerSize',10,...
    'LineWidth',2);
legend('RANDOM','BANJO 0%noise','BANJO 10%noise ','BANJO 30%noise ','fontsize',16)


% Compute the performance of random algorithm with take into account the
% prior knowledge (PK)
PK_f=0.3;

% Vector of reconstruction probability values
% Each value defines the probability that an element of the randomly 
% reconstructed matrix is zero
x_pr_v=linspace(0,1,100);

% Compute the corresponding vectors of PPV and Sens, for directed 
% randomly reconstructed networks

TP_dir=PK_f*(1-spar_coef_mean)+(1-PK_f)*(1-spar_coef_mean)*(1-x_pr_v);
%TN_dir=x_pr_v*spar_coef_mean;
FP_dir=(1-x_pr_v)*spar_coef_mean;
FN_dir=(1-PK_f)*x_pr_v*(1-spar_coef_mean);

sens_dir=TP_dir./(TP_dir+FN_dir);
% spec_dir=TN_dir./(TN_dir+FP_dir);
PPV_dir=TP_dir./(TP_dir+FP_dir);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%  PLOT RESULTS WITH PK  %%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure;
hold on;
grid on
if net_nodes==10
    title('n=10, 30%PK','FontSize', 16)
    axis([0.3 0.65 0.2 1]);
elseif net_nodes==20
    title('n=20, 30%PK','FontSize', 16)
    axis([0.3 0.5 0.2 1]);
else
    title('30%PK','FontSize', 16)
end
xlabel('Sn','FontSize', 14)
ylabel('PPV','FontSize', 14)
% plot the performance of the random algorithm
plot(sens_dir,PPV_dir,'r','LineWidth',2);

% plot the performance of BANJO, averaged of a 
% set of 20 networks, assuming 3 different level of noise: 

% 1) 0% level of noise
plot(mean(Se_Banjo(:,4)),mean(PPV_Banjo(:,4)),'ok','MarkerSize',10,...
    'LineWidth',2);

% 2) 10% level of noise
plot(mean(Se_Banjo(:,5)),mean(PPV_Banjo(:,5)),'sk','MarkerSize',10,...
    'LineWidth',2);

% 3) 30% level of noise
plot(mean(Se_Banjo(:,6)),mean(PPV_Banjo(:,6)),'dk','MarkerSize',10,...
    'LineWidth',2);

legend('RANDOM','BANJO 0%noise','BANJO 10%noise ','BANJO 30%noise ','fontsize',16)

eval(['PPV_Banjo_',num2str(net_nodes),'=PPV_Banjo;'])
eval(['Se_Banjo_',num2str(net_nodes),'=Se_Banjo;'])






