function [A_id,U_id,norm_est_err_dy,sol_found] = lmi_sign_constr_full_ex(Meas_mtx,sign_constr_list,null_b_list)

% The function computes the connectivity matrix A and exogenous
% perturbation vector B via convex optimization problem in the form of
% LMIs. The problem is solved taking into account the sign constraints to
% be imposed without setting any optimization variable to zero.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% INPUT VARIABLES
%
%
% Meas_mtx:  Measurements matrix, each column contains the measurements for
%            a node
%
%
% sign_constr_list: Matrix with 3 columns.
%             Constraints to be imposed on the matrix coefficients
%             first column --> row-index of the constrained coefficient
%             second column --> column-index of the constrained coefficient
%             third column --> sign of the constrained coefficient
%
% null_b_list: List that contains the indexes of the b-coefficients to be 
%              nullified          
% 
%
% OUTPUT VARIABLES
%
% A_id:            The identified connectivity matrix
%               
%
% U_id:            The identified input vector   
%               
% norm_est_err_dy: Estimation error generated by the
%                  identified model at each step 
%
% sol_found_cell:  A flag (true, if a solution has been found) 
%                 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



% determine the number of time points and nodes, respectively
[n_time_pts,n_nodes]=size(Meas_mtx);

% Matrix of the measurements at time point k from 1 to n_time_pts-1
Omega_st=Meas_mtx(1:n_time_pts-1,:)';
Omega_mtx=[Omega_st;ones(1,n_time_pts-1)];
% Matrix of the measurements at time point k+1 from 2 to n_time_pts
Theta_mtx=Meas_mtx(2:n_time_pts,:)';

eps_LMI=1e-6; % Small number, used in the LMI problem definition to enforce 
              % the positivity of the gevp right side
              
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%  LMI Problem Definition   %%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

setlmis([])
% Identity matrix, the columns will be used to generate the
% sign constraints inequalities
U_row=eye(n_nodes);
U_col=eye(n_nodes+1);
dec_var_vct=zeros(n_nodes,1);

% Matrix variable to be identified, LMIs solution

if size(null_b_list,1)==n_nodes
    [A_lmi n_A s_A]=lmivar(2, [n_nodes n_nodes]);
    % Matrix variable to be identified, LMIs solution
    X_lmi =lmivar(3,[s_A dec_var_vct]);
else
    [A_lmi n_A s_A]=lmivar(2, [n_nodes n_nodes]);
    if isempty(null_b_list)
        [B_lmi n_B s_B]=lmivar(2, [n_nodes 1]);
    else
        lmi_var_num=n_nodes*n_nodes;
        dec_var_vct=zeros(n_nodes,1);
        for idx1=1:n_nodes
            % Check if the element bi is equal to zero
            if not(sum(idx1==null_b_list))
                % No zero constraint has been found
                % Increment the scalar decision variables number
                lmi_var_num=lmi_var_num+1;
                dec_var_vct(idx1,1)=lmi_var_num;
            end
        end
        [B_lmi n_B s_B]=lmivar(3, dec_var_vct);
    end
    % Matrix variable to be identified, LMIs solution
    X_lmi =lmivar(3,[s_A,s_B]);
end

% Definition of the sign constraint LMIs
if not(isempty(sign_constr_list))
    for idx1=1:size(sign_constr_list,1) % Number of sign constraints
        row_cstr=sign_constr_list(idx1,1);
        col_cstr=sign_constr_list(idx1,2);
        type_cstr=sign_constr_list(idx1,3);

        if (type_cstr==1)
            % Positive coefficient constraint
            lmitag=newlmi;
            lmiterm([-lmitag 1 1 X_lmi], U_row(:,row_cstr)',U_col(:,col_cstr),'s');
        else
            % Negative coefficient constraint
            lmitag=newlmi;
            lmiterm([lmitag 1 1 X_lmi], U_row(:,row_cstr)',U_col(:,col_cstr),'s');
        end
    end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Stability condition for a discrete-time system x(k+1) = A x(k)
% The Lyapunov inequality A' P A - P < 0 is rewritten as
%
%
%                         | -P      A'P  |
%                         |              | < 0
%                         |  PA      -P  |
%
% In order to obtain a LMI we choose P = I
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
lmitag=newlmi;
lmiterm([lmitag 2 1 A_lmi],1,1);
lmiterm([lmitag 1 1 0],-1);
lmiterm([lmitag 2 2 0],-1);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Definition of the LMI for data fitting
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%        | -lambda*I             (Theta-Xlmi*Omega)'|
%        |                                          | < 0
%        | (Theta-Xlmi*Omega)             -I        |
%
% The LMI problem is defined as the following GEVP, minimizing lambda
%
%  |       0             (Theta-Xlmi*Omega)'|          | I      0     |
%  |                                        | < lambda |              |
%  | (Theta-Xlmi*Omega)             -I      |          | 0  eps_LMI*I |

% Specification of the positivity constraint of the right side for
% well-posedness
lmitag=newlmi;
lmiterm([-lmitag 1 1 0],1);
lmiterm([-lmitag 2 2 0], eps_LMI);

% Last LMI
lmitag=newlmi;
lmiterm([lmitag 2 2 0], -1);
lmiterm([lmitag 2 1 X_lmi], -1, Omega_mtx);
lmiterm([lmitag 2 1 0], Theta_mtx);
lmiterm([-lmitag 1 1 0], 1);
lmiterm([-lmitag 2 2 0], eps_LMI);

% Number of linear fractional constraints in the GEVP
nlfc=1;

% Get the LMIs
lmis=getlmis;
% The trace of execution is turned off
options(5)=1;
% Solve the GEVP
[lopt,xopt]=gevp(lmis,nlfc,options);

sol_found = not(isempty(xopt));
if not(sol_found)
   A_id=[]; 
   U_id=[];
   norm_est_err_dy=[];
else
    %  Assign the identified matrices 
    X_id=dec2mat(lmis, xopt ,X_lmi);
    A_id=X_id(1:n_nodes,1:n_nodes);
    U_id=X_id(1:n_nodes,n_nodes+1);
    
    % Compute the state evolution of the identified model and compare
    % with experimental measurements
    X0=Omega_st(1:n_nodes,1);
    Xsim_dy(:,1)=X0;
    for idx=2:n_time_pts
        Theta_sim= A_id*X0+U_id;
        Xsim_dy(:,idx)=Theta_sim;
        X0=Theta_sim;
    end
    % Compute the absolute error matrix
    Err_mtx_dy=abs(Meas_mtx-Xsim_dy');

    % Normalization of the error matrix
    % Find the maximum absolute value in the matrix
    max_Meas_mtx=max(max(abs(Meas_mtx)));
    % Divide each element by the abs value of the corresponding experimental
    % measurement
    for idx2=1:size(Meas_mtx,2)
        % The normalized error is computed only on the values that are at
        % least 10% of the maximum of the signal and only on those signals
        % whose maximum is at least 10% of the maximum over all the signals
        if max(abs(Meas_mtx(:,idx2)))>max_Meas_mtx*0.1;
            col_thr=0.1*max(abs(Meas_mtx(:,idx2)));
        else
            % Do not consider this column in the computation of the estimation
            % error
            col_thr=Inf;
        end
        for idx1=1:size(Meas_mtx,1)
            if abs(Meas_mtx(idx1,idx2))>col_thr
                Err_mtx_dy(idx1,idx2)=Err_mtx_dy(idx1,idx2)/abs(Meas_mtx(idx1,idx2));
            else
                Err_mtx_dy(idx1,idx2)=0;
            end
        end
    end
    % Compute the cumulative error for each signal, pick the largest one and
    % divide by the number of time points
    norm_est_err_dy=norm(Err_mtx_dy,1)/n_time_pts;
end
